@@ -28,7 +28,8 @@ __date__ = "04/06/2021"
 
 import numpy as np
 import matplotlib.pyplot as plt
-from scipy.ndimage import gaussian_filter
+from scipy.ndimage import (
+    gaussian_filter, gaussian_laplace, gaussian_gradient_magnitude)
 import h5py
 
 try:
@@ -38,7 +39,7 @@ except ImportError:
 
 
 class IsoScan:
-    """Sum of entry.data over the scan axis within [scan-dscan, scan+dscan]
+    """Sum of entry.data over the scan axis within [scan-dscan, scan+dscan].
 
     Args:
         entry (NavEntry class): The class for the data explored by NavARP;
@@ -91,6 +92,7 @@ class IsoScan:
         curvature=None,
         kbins=None
     ):
+        """Class initialization."""
         self.scan = scan
         self.dscan = dscan
         self.data_init = isocut.maps_sum(
@@ -122,7 +124,7 @@ class IsoScan:
         self.dataprocessing(norm_mode, sigma, order, curvature, kbins)
 
     def get_axs_and_data(self, xname='auto', yname='auto'):
-        """Return the data and relative axes
+        """Return the data and relative axes.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
@@ -130,7 +132,6 @@ class IsoScan:
         Returns:
             xax, yax, data, xname, yname
         """
-
         if self.kx_interp is not None:
             xname = 'kx'
             xax = self.kx_interp
@@ -159,7 +160,7 @@ class IsoScan:
         return xax, yax, self.data, xname, yname
 
     def dataprocessing(self, norm_mode, sigma, order, curvature, kbins):
-        """Use data post-processing procedure
+        """Apply data post-processing procedure.
 
         Args:
             norm_mode (string, optional, 'no' or 'all' or 'each'): if 'no', no
@@ -178,12 +179,12 @@ class IsoScan:
                 with that derivative of a Gaussian. For example, an order of 2
                 corresponds to convolution with that second derivative of a
                 Gaussian;
-            curvature (integer, optional, default=None): if not None the data
-                are the curvature of the signal;
+            curvature (float, optional, default=None): if not None the data
+                are the curvature of the signal, only in 1-dimension, meaning
+                only for sigma with one value;
             kbins (integer, optional, default=None): if not None the data are
                 interpolated on a uniform k-array binned with the kbins number.
         """
-
         if kbins is not None:
             if hasattr(self, 'kx'):
                 self.kx_interp, self.data = kinterp.get_isoscan(
@@ -194,18 +195,14 @@ class IsoScan:
             self.data = np.copy(self.data_init)
             self.kx_interp = None
 
-        if norm_mode != 'no':
+        if norm_mode == 'each':
             self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
         if sigma is not None:
-            if curvature is not None:
-                dev1pwr2 = gaussian_filter(self.data, sigma, order=1)**2
-                dev2 = gaussian_filter(self.data, sigma, order=2)
-                curv_max = curvature*abs(dev1pwr2.max() - dev1pwr2.min())
-                self.data = dev2*np.power((curv_max + dev1pwr2), -1.5)
+            self.data = postprocessing(self.data, sigma, order, curvature)
 
-            else:
-                self.data = gaussian_filter(self.data, sigma, order)
+        if norm_mode == 'all':
+            self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
     def show(
         self,
@@ -218,7 +215,7 @@ class IsoScan:
         print_out=False,
         cmapscale="linear"
     ):
-        """Plot the isomap
+        """Plot the isomap.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
@@ -236,7 +233,6 @@ class IsoScan:
             The returned value is the object matplotlib.collections.QuadMesh
                 from ax.pcolormesh.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
         if style is None:
@@ -255,28 +251,25 @@ class IsoScan:
         )
         return qmisoscan
 
-    def export_as_nxs(
+    def export_as_hdf5(
         self,
-        file_path_nxs,
+        file_path,
         xname='auto',
         yname='auto',
         title='NavARP_isoscan'
     ):
-        """Save the isomap as NXdata nexus class
+        """Save the isomap as hdf5 format.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
             yname (string, optional, auto/eef/ekin): Select the y variable;
             title (string, optiona): data title.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
-        f = h5py.File(file_path_nxs, "w")  # create the HDF5 NeXus file
-        f.attrs[u"default"] = u"isoscan"
+        f = h5py.File(file_path, "w")  # create the HDF5 file
 
         nxdata = f.create_group(u"isoscan")
-        nxdata.attrs[u"NX_class"] = u"NXdata"
 
         nxdata.create_dataset(u"title", data=title)
 
@@ -309,21 +302,80 @@ class IsoScan:
 
         f.close()
 
+    def export_as_nxs(
+        self,
+        file_path,
+        xname='auto',
+        yname='auto',
+        title='NavARP_isoscan'
+    ):
+        """Save the isomap as NXdata nexus class.
+
+        Args:
+            xname (string, optional, auto/tht/kx): Select the x variable;
+            yname (string, optional, auto/eef/ekin): Select the y variable;
+            title (string, optiona): data title.
+        """
+        xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
+
+        if len(xax.shape) > 1:
+            raise ValueError(
+                "Data must be interpolated on uniform grid, first. "
+                "Use kbins during isoscan initialization or in dataprocessing."
+            )
+            return None
+
+        f = h5py.File(file_path, "w")  # create the HDF5 NeXus file
+        f.attrs[u"default"] = u"isoscan"
+
+        nxdata = f.create_group(u"isoscan")
+        nxdata.attrs[u"NX_class"] = u"NXdata"
+
+        nxdata.create_dataset(u"title", data=title)
+
+        if xname == 'kx':
+            axes_names = ['', 'momentum']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
+            xdata.attrs[u"units"] = u"1/angstrom"
+        else:
+            axes_names = ['', 'angles']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
+            xdata.attrs[u"units"] = u"degrees"
+        xdata.attrs[u"axis"] = 2
+        xdata.attrs[u"primary"] = 1
+
+        if yname == 'eef':
+            axes_names[0] = "binding_energies"
+        else:
+            axes_names[0] = "kinetic_energies"
+        ydata = nxdata.create_dataset(axes_names[0], data=yax)
+        ydata.attrs[u"axis"] = 1
+        ydata.attrs[u"primary"] = 1
+        ydata.attrs[u"units"] = u"eV"
+
+        data_nx = nxdata.create_dataset(
+            u"data", data=data.transpose(), compression='gzip', chunks=True)
+        data_nx.attrs[u"units"] = u"counts"
+        nxdata.attrs[u"signal"] = u"data"
+
+        nxdata.attrs[u"axes"] = [axesn.encode('utf8') for axesn in axes_names]
+
+        f.close()
+
     def export_as_itx(
         self,
-        file_path_itx,
+        file_path,
         xname='auto',
         yname='auto',
         wave_name='NavARP_isoscan'
     ):
-        """Save the isomap as Igor Text file
+        """Save the isomap as Igor Pro Text file.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
             yname (string, optional, auto/eef/ekin): Select the y variable;
             wave_name (string, optiona): name of the wave.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
         if len(xax.shape) > 1:
@@ -361,7 +413,7 @@ class IsoScan:
         )
 
         np.savetxt(
-            file_path_itx,
+            file_path,
             data,
             fmt='%.6f',
             delimiter=' ',
@@ -374,7 +426,7 @@ class IsoScan:
 
 
 class IsoEnergy:
-    """Sum of entry.data over the ebin axis within [ebin-debin, ebin+debin]
+    """Sum of entry.data over the ebin axis within [ebin-debin, ebin+debin].
 
     Args:
         entry (NavEntry class): The class for the data explored by NavARP;
@@ -428,6 +480,7 @@ class IsoEnergy:
         curvature=None,
         kbins=None
     ):
+        """Class initialization."""
         self.ebin = ebin
         self.debin = debin
         self.data_init = isocut.maps_sum(
@@ -449,7 +502,7 @@ class IsoEnergy:
         self.dataprocessing(norm_mode, sigma, order, curvature, kbins)
 
     def dataprocessing(self, norm_mode, sigma, order, curvature, kbins):
-        """Use data post-processing procedure
+        """Apply data post-processing procedure.
 
         Args:
             norm_mode (string, optional, 'no' or 'all' or 'each'): if 'no', no
@@ -468,8 +521,9 @@ class IsoEnergy:
                 with that derivative of a Gaussian. For example, an order of 2
                 corresponds to convolution with that second derivative of a
                 Gaussian;
-            curvature (integer, optional, default=None): if not None the data
-                are the curvature of the signal;
+            curvature (float, optional, default=None): if not None the data
+                are the curvature of the signal, only in 1-dimension, meaning
+                only for sigma with one value;
             kbins (array, optional, default=None): if not None, the data are
                 interpolated on a uniform k-array binned following the
                 [kxbins, kybins] numbers; kbins must be an array of two integer
@@ -478,7 +532,6 @@ class IsoEnergy:
                 is necessary due to how the code recognize the element order
                 and placement in the data matrix).
         """
-
         if kbins is not None:
             if len(kbins) != 2:
                 raise ValueError(
@@ -489,6 +542,8 @@ class IsoEnergy:
                     kbins[1] += 1
                 self.kx_interp, self.ks_interp, self.data = kinterp.get_isoen(
                     kbins[0], kbins[1], self.kx, self.ks, self.data_init)
+                if self.ks_interp.shape[0] == self.data.shape[1]:
+                    self.data = self.data.transpose()
             else:
                 raise AttributeError('No valid tht_an, do set_tht_an first')
         else:
@@ -496,21 +551,17 @@ class IsoEnergy:
             self.kx_interp = None
             self.ks_interp = None
 
-        if norm_mode != 'no':
+        if norm_mode == 'each':
             self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
         if sigma is not None:
-            if curvature is not None:
-                dev1pwr2 = gaussian_filter(self.data, sigma, order=1)**2
-                dev2 = gaussian_filter(self.data, sigma, order=2)
-                curv_max = curvature*abs(dev1pwr2.max() - dev1pwr2.min())
-                self.data = dev2*np.power((curv_max + dev1pwr2), -1.5)
+            self.data = postprocessing(self.data, sigma, order, curvature)
 
-            else:
-                self.data = gaussian_filter(self.data, sigma, order)
+        if norm_mode == 'all':
+            self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
     def get_axs_and_data(self, xname='auto', yname='auto'):
-        """Return the data and relative axes
+        """Return the data and relative axes.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
@@ -518,7 +569,6 @@ class IsoEnergy:
         Returns:
             xax, yax, data, xname, yname
         """
-
         if self.kx_interp is not None:
             xname = 'kx'
             xax = self.kx_interp
@@ -543,7 +593,7 @@ class IsoEnergy:
             else:
                 xax = self.angles
 
-            if yname == 'auto':
+            if yname == 'auto' or (yname == 'ks'):
                 if hasattr(self, 'ks'):
                     if self.scan_type == 'hv':
                         yname = 'kz'
@@ -574,9 +624,10 @@ class IsoEnergy:
         z_range=None,
         style=None,
         print_out=False,
-        cmapscale="linear"
+        cmapscale="linear",
+        rotate=None
     ):
-        """Plot the isomap
+        """Plot the isomap.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
@@ -589,21 +640,37 @@ class IsoEnergy:
             print_out (boolean, optional): If True, print the axes dimension
                 with them adaptations;
             cmapscale (string, optional, linear/log/power): Select the scale
-                mode.
+                mode;
+            rotate (float, optional): angle in degrees for a counter clockwise
+                rotation of the shown isoenergy.
         Returns:
             The returned value is the object matplotlib.collections.QuadMesh
                 from ax.pcolormesh.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
         if style is None:
             style = xname + '_' + yname
 
+        if rotate:
+            if len(xax.shape) != 2:
+                xax_cp = np.tile(xax, (self.scans.shape[0], 1))
+            else:
+                xax_cp = np.copy(xax)
+
+            if len(yax.shape) != 2:
+                yax_cp = np.tile(yax, (self.angles.shape[0], 1)).T
+            else:
+                yax_cp = np.copy(yax)
+
+            rot_ang = np.deg2rad(rotate)
+            xax = np.cos(rot_ang)*xax_cp - np.sin(rot_ang)*yax_cp
+            yax = np.sin(rot_ang)*xax_cp + np.cos(rot_ang)*yax_cp
+
         qmiso = navplt.pimage(
             xax,
             yax,
-            self.data,
+            data,
             cmap=cmap,
             ax=ax,
             z_range=z_range,
@@ -613,54 +680,51 @@ class IsoEnergy:
         )
         return qmiso
 
-    def export_as_nxs(
+    def export_as_hdf5(
         self,
-        file_path_nxs,
+        file_path,
         xname='auto',
         yname='auto',
         title='NavARP_isoenergy'
     ):
-        """Save the isomap as NXdata nexus class
+        """Save the isomap as hdf5 format.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
             yname (string, optional, auto/eef/ekin): Select the y variable;
             title (string, optiona): data title.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
-        f = h5py.File(file_path_nxs, "w")  # create the HDF5 NeXus file
-        f.attrs[u"default"] = u"isoenergy"
+        f = h5py.File(file_path, "w")  # create the HDF5 NeXus file
 
         nxdata = f.create_group(u"isoenergy")
-        nxdata.attrs[u"NX_class"] = u"NXdata"
 
         nxdata.create_dataset(u"title", data=title)
 
         if xname == 'kx':
-            axes_names = ['k_along_slit', '']
-            xdata = nxdata.create_dataset(axes_names[0], data=xax)
+            axes_names = ['', 'k_along_slit']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
             xdata.attrs[u"units"] = u"1/angstrom"
         else:
-            axes_names = ['angles', '']
-            xdata = nxdata.create_dataset(axes_names[0], data=xax)
+            axes_names = ['', 'angles']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
             xdata.attrs[u"units"] = u"degrees"
-        xdata.attrs[u"axis"] = 1
+        xdata.attrs[u"axis"] = 2
         xdata.attrs[u"primary"] = 1
 
         if (yname == 'ky') or (yname == 'kz'):
-            axes_names[1] = 'k_along_scan'
-            ydata = nxdata.create_dataset(axes_names[0], data=xax)
+            axes_names[0] = 'k_along_scan'
+            ydata = nxdata.create_dataset(axes_names[0], data=yax)
             ydata.attrs[u"units"] = u"1/angstrom"
         else:
-            axes_names[1] = self.scan_type
-            ydata = nxdata.create_dataset(axes_names[0], data=xax)
+            axes_names[0] = self.scan_type
+            ydata = nxdata.create_dataset(axes_names[0], data=yax)
             if self.scan_type == 'hv':
                 ydata.attrs[u"units"] = u"eV"
             else:
                 ydata.attrs[u"units"] = u"degree"
-        ydata.attrs[u"axis"] = 2
+        ydata.attrs[u"axis"] = 1
         ydata.attrs[u"primary"] = 1
 
         data_nx = nxdata.create_dataset(
@@ -668,25 +732,24 @@ class IsoEnergy:
         data_nx.attrs[u"units"] = u"counts"
         nxdata.attrs[u"signal"] = u"data"
 
-        nxdata.attrs[u"axes"] = [axesn.encode('utf8') for axesn in axes_names]
+        nxdata.attrs[u"axes"] = axes_names
 
         f.close()
 
-    def export_as_itx(
+    def export_as_nxs(
         self,
-        file_path_itx,
+        file_path,
         xname='auto',
         yname='auto',
-        wave_name='NavARP_isoenergy'
+        title='NavARP_isoenergy'
     ):
-        """Save the isomap as Igor Text file
+        """Save the isomap as NXdata nexus class.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
             yname (string, optional, auto/eef/ekin): Select the y variable;
-            wave_name (string, optiona): name of the wave.
+            title (string, optiona): data title.
         """
-
         xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
 
         if (len(xax.shape) > 1) or (len(yax.shape) > 1):
@@ -696,6 +759,74 @@ class IsoEnergy:
             )
             return None
 
+        f = h5py.File(file_path, "w")  # create the HDF5 NeXus file
+        f.attrs[u"default"] = u"isoenergy"
+
+        nxdata = f.create_group(u"isoenergy")
+        nxdata.attrs[u"NX_class"] = u"NXdata"
+
+        nxdata.create_dataset(u"title", data=title)
+
+        if xname == 'kx':
+            axes_names = [u'', u'k_along_slit']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
+            xdata.attrs[u"units"] = u"1/angstrom"
+        else:
+            axes_names = [u'', u'angles']
+            xdata = nxdata.create_dataset(axes_names[1], data=xax)
+            xdata.attrs[u"units"] = u"degrees"
+        xdata.attrs[u"axis"] = 2
+        xdata.attrs[u"primary"] = 1
+
+        if (yname == 'ky') or (yname == 'kz'):
+            axes_names[0] = u'k_along_scan'
+            ydata = nxdata.create_dataset(axes_names[0], data=yax)
+            ydata.attrs[u"units"] = u"1/angstrom"
+        else:
+            axes_names[0] = self.scan_type
+            ydata = nxdata.create_dataset(axes_names[0], data=yax)
+            if self.scan_type == u'hv':
+                ydata.attrs[u"units"] = u"eV"
+            else:
+                ydata.attrs[u"units"] = u"degree"
+        ydata.attrs[u"axis"] = 1
+        ydata.attrs[u"primary"] = 1
+
+        data_nx = nxdata.create_dataset(
+            u"data", data=data, compression='gzip', chunks=True)
+        data_nx.attrs[u"units"] = u"counts"
+        nxdata.attrs[u"signal"] = u"data"
+
+        nxdata.attrs[u"axes"] = axes_names
+
+        f.close()
+
+    def export_as_itx(
+        self,
+        file_path,
+        xname='auto',
+        yname='auto',
+        wave_name='NavARP_isoenergy'
+    ):
+        """Save the isomap as Igor Pro Text file.
+
+        Args:
+            xname (string, optional, auto/tht/kx): Select the x variable;
+            yname (string, optional, auto/eef/ekin): Select the y variable;
+            wave_name (string, optiona): name of the wave.
+        """
+        xax, yax, data, xname, yname = self.get_axs_and_data(xname, yname)
+
+        if (len(xax.shape) > 1) or (len(yax.shape) > 1):
+            raise ValueError(
+                "Data must be interpolated on uniform grid, first. Use "
+                "kbins during isoenergy initialization or in dataprocessing."
+            )
+            return None
+
+        # data transpose for itx
+        data_itx = data.transpose()
+
         if xname == 'kx':
             xlabel = 'k along slit [1/Angstrom]'
         else:
@@ -711,7 +842,7 @@ class IsoEnergy:
 
         header = (
             "IGOR\n" +
-            "WAVES/N=({0[0]},{0[1]})\t".format(data.shape) +
+            "WAVES/N=({0[0]},{0[1]})\t".format(data_itx.shape) +
             wave_name + "\n" + "BEGIN"
         )
 
@@ -727,8 +858,8 @@ class IsoEnergy:
         )
 
         np.savetxt(
-            file_path_itx,
-            data,
+            file_path,
+            data_itx,
             fmt='%.6f',
             delimiter=' ',
             newline='\n',
@@ -740,7 +871,7 @@ class IsoEnergy:
 
 
 class IsoAngle:
-    """Sum of entry.data over the angle axis in [angle-dangle, angle+dangle]
+    """Sum of entry.data over the angle axis in [angle-dangle, angle+dangle].
 
     Args:
         entry (NavEntry class): The class for the data explored by NavARP;
@@ -791,6 +922,7 @@ class IsoAngle:
         curvature=None,
         kbins=None
     ):
+        """Class initialization."""
         self.angle = angle
         self.dangle = dangle
         self.data_init = isocut.maps_sum(
@@ -821,7 +953,7 @@ class IsoAngle:
         print_out=False,
         cmapscale="linear"
     ):
-        """Plot the isomap
+        """Plot the isomap.
 
         Args:
             xname (string, optional, auto/phi/hv): Select the x variable;
@@ -840,26 +972,30 @@ class IsoAngle:
                 from ax.pcolormesh.
 
         """
-
-        if xname == 'auto':
+        if xname == 'auto' or (xname == 'scan'):
             if self.scan_type == 'hv':
                 xname = 'hv'
             else:
                 xname = 'phi'
-        x_ax = self.scans
+            x_ax = self.scans
+        elif xname == 'ekin':
+            x_ax = self.ekins
+        elif xname == 'eef':
+            x_ax = self.ebins
 
         if yname == 'auto':
             yname = 'eef'
             y_ax = self.ebins
         elif yname == 'ekin':
-            if hasattr(self, 'ekin'):
-                y_ax = self.ekins
-            else:
-                print('Attribute ekin does not exist, using ebin')
-                y_ax = self.ebins
-                yname = 'eef'
+            y_ax = self.ekins
         elif yname == 'eef':
             y_ax = self.ebins
+        elif yname == 'scan':
+            if self.scan_type == 'hv':
+                yname = 'hv'
+            else:
+                yname = 'phi'
+            y_ax = self.scans
 
         if style is None:
             style = xname + '_' + yname
@@ -878,7 +1014,7 @@ class IsoAngle:
         return qmisoangle
 
     def dataprocessing(self, norm_mode, sigma, order, curvature):
-        """Use data post-processing procedure
+        """Use data post-processing procedure.
 
         Args:
             norm_mode (string, optional, 'no' or 'all' or 'each'): if 'no', no
@@ -900,25 +1036,20 @@ class IsoAngle:
             curvature (integer, optional, default=None): if not None the data
                 are the curvature of the signal.
         """
-
         self.data = np.copy(self.data_init)
 
-        if norm_mode != 'no':
+        if norm_mode == 'each':
             self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
         if sigma is not None:
-            if curvature is not None:
-                dev1pwr2 = gaussian_filter(self.data, sigma, order=1)**2
-                dev2 = gaussian_filter(self.data, sigma, order=2)
-                curv_max = curvature*abs(dev1pwr2.max() - dev1pwr2.min())
-                self.data = dev2*np.power((curv_max + dev1pwr2), -1.5)
+            self.data = postprocessing(self.data, sigma, order, curvature)
 
-            else:
-                self.data = gaussian_filter(self.data, sigma, order)
+        if norm_mode == 'all':
+            self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
 
 class IsoK:
-    """Interpolate the entry.data along specified k-points
+    """Interpolate the entry.data along specified k-points.
 
     Args:
         entry (NavEntry class): the class for the data explored by NavARP;
@@ -950,8 +1081,9 @@ class IsoK:
             with that derivative of a Gaussian. For example, an order of 2
             corresponds to convolution with that second derivative of a
             Gaussian;
-        curvature (integer, optional, default=None): if not None the data
-            are the curvature of the signal.
+        curvature (float, optional, default=None): if not None the data
+            are the curvature of the signal, only in 1-dimension, meaning
+            only for sigma with one value.
 
     Attributes:
         data_init (ndarray): The matrix composed by interpolated values along
@@ -988,7 +1120,7 @@ class IsoK:
         order=2,
         curvature=None
     ):
-
+        """Class initialization."""
         self.klabels = klabels
         self.kx_pts = kx_pts
         self.ky_pts = ky_pts
@@ -1016,7 +1148,7 @@ class IsoK:
         self.dataprocessing(norm_mode, sigma, order, curvature)
 
     def dataprocessing(self, norm_mode, sigma, order, curvature):
-        """Use data post-processing procedure
+        """Use data post-processing procedure.
 
         Args:
             norm_mode (string, optional, 'no' or 'all' or 'each'): if 'no', no
@@ -1038,31 +1170,25 @@ class IsoK:
             curvature (integer, optional, default=None): if not None the data
                 are the curvature of the signal.
         """
-
         self.data = np.copy(self.data_init)
 
-        if norm_mode != 'no':
+        if norm_mode == 'each':
             self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
         if sigma is not None:
-            if curvature is not None:
-                dev1pwr2 = gaussian_filter(self.data, sigma, order=1)**2
-                dev2 = gaussian_filter(self.data, sigma, order=2)
-                curv_max = curvature*abs(dev1pwr2.max() - dev1pwr2.min())
-                self.data = dev2*np.power((curv_max + dev1pwr2), -1.5)
+            self.data = postprocessing(self.data, sigma, order, curvature)
 
-            else:
-                self.data = gaussian_filter(self.data, sigma, order)
+        if norm_mode == 'all':
+            self.data = navplt.norm(self.data, mode=norm_mode, axis=1)
 
     def get_yax(self, yname='auto'):
-        """Return the selected y axis
+        """Return the selected y axis.
 
         Args:
             yname (string, optional, auto/eef/ekin): Select the y variable.
         Returns:
             yax, yname
         """
-
         if yname == 'auto':
             yname = 'eef'
             yax = self.ebins
@@ -1080,7 +1206,7 @@ class IsoK:
 
     def path_show(
             self, ax=None, lc='k', textcolor='r', textsize=14, xytext=(8, 5)):
-        """Plot the path
+        """Plot the path.
 
         Args:
             ax (matplotlib.axes, optional): Axes, if None it is created inside
@@ -1091,7 +1217,6 @@ class IsoK:
             xytext (array, optional): relative text position of the labels.
 
         """
-
         if ax is None:
             fig, ax = plt.subplots(1)
 
@@ -1137,7 +1262,7 @@ class IsoK:
         ls='-',
         lw=1
     ):
-        """Plot the isomap
+        """Plot the isomap.
 
         Args:
             yname (string, optional, auto/eef/ekin): Select the y variable;
@@ -1155,7 +1280,6 @@ class IsoK:
                 from ax.pcolormesh.
 
         """
-
         xname = 'kx'
         xax = self.krho
 
@@ -1198,20 +1322,19 @@ class IsoK:
 
     def export_as_nxs(
         self,
-        file_path_nxs,
+        file_path,
         yname='auto',
         title='NavARP_isok'
     ):
-        """Save the isomap as NXdata nexus class
+        """Save the isomap as NXdata nexus class.
 
         Args:
             yname (string, optional, auto/eef/ekin): Select the y variable;
             title (string, optiona): data title.
         """
-
         yax, yname = self.get_yax(yname)
 
-        f = h5py.File(file_path_nxs, "w")  # create the HDF5 NeXus file
+        f = h5py.File(file_path, "w")  # create the HDF5 NeXus file
         f.attrs[u"default"] = u"isok"
 
         nxdata = f.create_group(u"isok")
@@ -1219,23 +1342,27 @@ class IsoK:
 
         nxdata.create_dataset(u"title", data=title)
 
-        axes_names = ['momentum', '']
-        xdata = nxdata.create_dataset(axes_names[0], data=self.krho)
+        axes_names = ['', 'momentum']
+        xdata = nxdata.create_dataset(axes_names[1], data=self.krho)
         xdata.attrs[u"units"] = u"1/angstrom"
-        xdata.attrs[u"axis"] = 1
+        xdata.attrs[u"axis"] = 2
         xdata.attrs[u"primary"] = 1
 
         if yname == 'eef':
-            axes_names[1] = "binding_energies"
+            axes_names[0] = "binding_energies"
         else:
-            axes_names[1] = "kinetic_energies"
-        ydata = nxdata.create_dataset(axes_names[1], data=yax)
-        ydata.attrs[u"axis"] = 2
+            axes_names[0] = "kinetic_energies"
+        ydata = nxdata.create_dataset(axes_names[0], data=yax)
+        ydata.attrs[u"axis"] = 1
         ydata.attrs[u"primary"] = 1
         ydata.attrs[u"units"] = u"eV"
 
         data_nx = nxdata.create_dataset(
-            u"data", data=self.data, compression='gzip', chunks=True)
+            u"data",
+            data=self.data.transpose(),
+            compression='gzip',
+            chunks=True
+        )
         data_nx.attrs[u"units"] = u"counts"
         nxdata.attrs[u"signal"] = u"data"
 
@@ -1245,18 +1372,17 @@ class IsoK:
 
     def export_as_itx(
         self,
-        file_path_itx,
+        file_path,
         yname='auto',
         wave_name='NavARP_isok'
     ):
-        """Save the isomap as Igor Text file
+        """Save the isomap as Igor Pro Text file.
 
         Args:
             xname (string, optional, auto/tht/kx): Select the x variable;
             yname (string, optional, auto/eef/ekin): Select the y variable;
             wave_name (string, optiona): name of the wave.
         """
-
         xax = self.krho
         yax, yname = self.get_yax(yname)
 
@@ -1283,7 +1409,7 @@ class IsoK:
         )
 
         np.savetxt(
-            file_path_itx,
+            file_path,
             self.data,
             fmt='%.6f',
             delimiter=' ',
@@ -1293,3 +1419,23 @@ class IsoK:
             comments='',
             encoding=None
         )
+
+
+def postprocessing(data, sigma, order, curvature):
+    """Post-processing procedure on data."""
+    if order == 0:
+        return gaussian_filter(data, sigma, order)
+    elif order == 1:
+        return data**3/gaussian_gradient_magnitude(data, sigma)
+    elif order == 2:
+        if isinstance(sigma, (list, tuple, np.ndarray)):
+            return gaussian_laplace(data, sigma)
+        else:
+            if curvature is not None:
+                dev1pwr2 = gaussian_filter(data, sigma, order=1)**2
+                dev2 = gaussian_filter(data, sigma, order=2)
+                curv_max = curvature*abs(
+                    np.nanmax(dev1pwr2) - np.nanmin(dev1pwr2))
+                return dev2*np.power((curv_max + dev1pwr2), -1.5)
+            else:
+                return gaussian_filter(data, sigma, order)
