@@ -441,7 +441,7 @@ def get_isoscan(
     isoscan,
     kbins,
     kind="cubic",
-    fill_value="extrapolate",
+    fill_value=np.nan,
     assume_sorted=True
 ):
     """Interpolate isoscan on the generated uniform binned k vector
@@ -500,7 +500,7 @@ def get_isoscan_from_kx_interp(
     energies,
     isoscan,
     kind="nearest",
-    fill_value="extrapolate",
+    fill_value=np.nan,
     assume_sorted=True
 ):
     """Interpolate isoscan on the k vector
