@@ -53,6 +53,10 @@ except ImportError:
 
 
 def decode_h5py(string):
+    """Get only the string in the case of np.ndarray or byte."""
+    if isinstance(string, np.ndarray):
+        string = string[0]
+
     try:
         string = string.decode("utf-8")
     except (UnicodeDecodeError, AttributeError):
@@ -61,7 +65,7 @@ def decode_h5py(string):
 
 
 class NavAnalyzer:
-    """NavAnalyzer is the class for the analyzer and its geometry
+    """NavAnalyzer is the class for the analyzer and its geometry.
 
     NavAnalyzer is defined from the NavEntry, and it considers the experimental
     set-ups of Lorea/ALBA(ES), I05/Diamond(GB), SXARPES-ADRESS/PSI(CH) and
@@ -73,8 +77,6 @@ class NavAnalyzer:
         phi_ap (float): Angle between analyzer axis (a) and photons (p) along
             the plane perpendicular to the slit (phi, so the name "phi_ap");
         work_fun (float): analyzer work function;
-        deflector (Boolean): If True, the analyzer is using a deflector, so it
-            can collect electrons out from the slit plane;
         scan_type (str): Acquisition method.
 
         _set_def_lorea_alba: Set defaul values for Lorea/ALBA(ES)
@@ -83,54 +85,47 @@ class NavAnalyzer:
         _set_def_antares_soleil: Set defaul values for Antares/Soleil(FR)
     """
 
-    def __init__(self, tht_ap=50, phi_ap=0, work_fun=4.5, deflector=False):
+    def __init__(self, tht_ap=50, phi_ap=0, work_fun=4.5):
         self.tht_ap = tht_ap
         self.phi_ap = phi_ap
         self.work_fun = work_fun
-        self.deflector = deflector
 
     def get_attr(self):
-        return self.tht_ap, self.phi_ap, self.work_fun, self.deflector
+        return self.tht_ap, self.phi_ap, self.work_fun
 
-    def set_attr(self, tht_ap, phi_ap, work_fun, deflector):
+    def set_attr(self, tht_ap, phi_ap, work_fun):
         self.tht_ap = tht_ap
         self.phi_ap = phi_ap
         self.work_fun = work_fun
-        self.deflector = deflector
 
     def _set_def_lorea_alba(self):
         self.tht_ap = 55
         self.phi_ap = 0
         self.work_fun = 4.6
-        self.deflector = True
 
     def _set_def_i05_diamond(self):
         self.tht_ap = 0
         self.phi_ap = 50
         self.work_fun = 4.5
-        self.deflector = False
 
     def _set_def_sxarpes_psi(self):
         self.tht_ap = -70
         self.phi_ap = 0
         self.work_fun = 4.5
-        self.deflector = False
 
     def _set_def_antares_soleil(self):
         self.tht_ap = 55
         self.phi_ap = 0
         self.work_fun = 4.5
-        self.deflector = True
 
     def _set_def_cassiopee_soleil(self):
         self.tht_ap = 0
         self.phi_ap = 50
         self.work_fun = 4.5
-        self.deflector = False
 
 
 class NavEntry:
-    """NavEntry is the class for the data to be explored by NavARP
+    """NavEntry is the class for the data to be explored by NavARP.
 
     Args:
         scans (ndarray): scan axis of the acquisition method;
@@ -184,10 +179,19 @@ class NavEntry:
     def __init__(self, scans, angles, energies, data, scan_type, hv,
                  defl_angles=0, analyzer=NavAnalyzer(),
                  file_note="", file_path=""):
+        """Class initialization."""
         self.scans = scans
-        self.angles = angles
         self.energies = energies
-        self.data = data
+        # energies and angles cannot have the same dimension otherwise the
+        # code cannot recognize automatically recognize them in the plots
+        # showing sometimes transposed data showing wrongly exchanged axes
+        # so check if angles and energies has the same length
+        if len(angles) == len(energies):  # if true remove last angle value
+            self.angles = angles[:-1]
+            self.data = data[:, :-1, :]
+        else:
+            self.angles = angles
+            self.data = data
         self.scan_type = scan_type
         self.hv = hv
         self.defl_angles = defl_angles
@@ -202,8 +206,7 @@ class NavEntry:
         self.valid_kspace = False
 
     def init_ebins(self):
-        """Initialize the fermi level (efermi) and the binding enrgies (ebins).
-        """
+        """Initialize fermi level (efermi) and binding energies (ebins)."""
         # get fermi level and binding energies
         if (self.scan_type == "hv") or (self.scan_type == "repeated"):
             self.energies_init = np.copy(self.energies)
@@ -258,13 +261,13 @@ class NavEntry:
             self.ebins = self.energies - self.efermi
 
     def autoset_efermi(
-            self,
-            energy_range=None,
-            angle_range=None,
-            scan_range=None,
-            print_out=True
+        self,
+        energy_range=None,
+        angle_range=None,
+        scan_range=None,
+        print_out=True
     ):
-        """Find fermi level using logaritmic derivative and then fit
+        """Find fermi level using logaritmic derivative and then fit.
 
         Args:
             energy_range (ndarray): Energy range where to find fermi level
@@ -275,7 +278,6 @@ class NavEntry:
                 [min, max];
             print_out (boolean, optional): If True, print the obtained values.
         """
-
         self.efermi_fit_input = [energy_range, angle_range, scan_range]
         if angle_range is not None:
             angle_ind = np.where(
@@ -355,15 +357,14 @@ class NavEntry:
                 )
 
     def plt_efermi_fit(self, axfit=None, scan_i=0):
-        """Plot the fermi level fit result
+        """Plot the fermi level fit result.
 
         Args:
             axfit (matplotlib.axes, optional): Axes, if None it is created
-                inside the function
+                inside the function;
             scan_i  (integer): scans index, in the case of efermi fit for
-                each image along the scan axis;
+                each image along the scan axis.
         """
-
         try:
             [energy_range, angle_range, scan_range] = self.efermi_fit_input
         except AttributeError:
@@ -438,7 +439,7 @@ class NavEntry:
         hv_p=None,
         print_out=True
     ):
-        """Calculates angle between analyzer axis (a) and normal (n) to the
+        """Set angle between analyzer axis (a) and normal (n) to the
             surface along the plane of the slit (theta, so the name "tht_an").
 
         Note:
@@ -458,7 +459,6 @@ class NavEntry:
                 requested if p_hv==True;
             print_out (boolean, optional): If True, print the obtained values.
         """
-
         if hv_p is None:
             hv_p = self.hv
 
@@ -488,7 +488,7 @@ class NavEntry:
             k_perp_slit_for_kz=0,
             print_out=True
     ):
-        """Calculates tht_an, phi_an, scans_0 and phi.
+        """Set tht_an, phi_an, scans_0 and phi.
 
         Args:
             tht_p (float): Angular value of the reference point along the
@@ -505,7 +505,6 @@ class NavEntry:
                 requested if p_hv==True;
             print_out (boolean, optional): If True, print the obtained values.
         """
-
         if hv_p is None:
             hv_p = self.hv
 
@@ -655,12 +654,11 @@ class NavEntry:
                 curvature
             )
         else:
-            raise AttributeError(
-                'No valid kspace, do set_kspace() first')
+            raise AttributeError('No valid kspace, do set_kspace() first')
 
 
 def load(file_path):
-    """The function define NavEntry from file_path.
+    """Load the NavEntry from file_path.
 
     The function loads entry from:
         * NXarpes file from LOREA/ALBA(ES) and I05/Diamond(GB);
@@ -702,7 +700,6 @@ def load(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     error_msg = (
         'Data format error.\n' +
         'Unknown file structure, the only supported data are:\n' +
@@ -797,7 +794,7 @@ def load(file_path):
 
 
 def load_sxarpes_adress(h5f, file_path):
-    """Load data from SXARPES-ADRESS/PSI(CH)
+    """Load data from SXARPES-ADRESS/PSI(CH).
 
     Args:
         h5f: The HDF5 file-object from h5py.File.
@@ -806,14 +803,12 @@ def load_sxarpes_adress(h5f, file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
     analyzer = NavAnalyzer()
     analyzer._set_def_sxarpes_psi()
 
     # deflector angles
-    if not analyzer.deflector:
-        defl_angles = 0
+    defl_angles = 0
 
     # data
     h5f_data = h5f["/Matrix"]
@@ -921,7 +916,7 @@ def load_sxarpes_adress(h5f, file_path):
 
 
 def load_nxarpes(h5f, file_path):
-    """Load NXARPES data from LOREA/ALBA(ES) and I05/Diamond(GB)
+    """Load NXARPES data from LOREA/ALBA(ES) and I05/Diamond(GB).
 
     Args:
         h5f: The HDF5 file-object from h5py.File.
@@ -929,7 +924,6 @@ def load_nxarpes(h5f, file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
     analyzer = NavAnalyzer()
     instrument_name = h5f["entry1/instrument/name"][()]
@@ -947,9 +941,6 @@ def load_nxarpes(h5f, file_path):
         return entry
 
     h5fdic = {
-        'pol': (
-            "entry1/instrument/insertion_device/beam/final_polarisation_label"
-        ),
         'hv': "entry1/instrument/monochromator/energy",
         'slit': "entry1/instrument/monochromator/exit_slit_size",
         'lens_mode': "entry1/instrument/analyser/lens_mode",
@@ -960,7 +951,6 @@ def load_nxarpes(h5f, file_path):
         'x': "entry1/instrument/manipulator/sax",
         'y': "entry1/instrument/manipulator/say",
         'z': "entry1/instrument/manipulator/saz",
-        't_frame': "entry1/instrument/analyser/time_for_frames",
         't_channel': "entry1/instrument/analyser/time_per_channel",
     }
 
@@ -968,11 +958,21 @@ def load_nxarpes(h5f, file_path):
     for key in h5fdic:
         params[key] = h5f[h5fdic[key]][()]
 
+    # polarisation angles
+    if "lorea" in instrument_name:
+        params['pol'] = h5f[
+            "entry1/instrument/insertion_device/beam/final_polarisation"
+        ][()]
+    elif "i05" in instrument_name:
+        params['pol'] = h5f[
+            "entry1/instrument/insertion_device/beam/final_polarisation_label"
+        ][()]
+
     # deflector angles
-    if not analyzer.deflector:
-        params['deflector'] = np.array([0.])
-    else:
+    if "lorea" in instrument_name:
         params['deflector'] = h5f["entry1/instrument/analyser/defl_angles"][()]
+    elif "i05" in instrument_name:
+        params['deflector'] = np.array([0.])
 
     # angles
     angles = h5f["entry1/instrument/analyser/angles"][()]
@@ -1041,7 +1041,7 @@ def load_nxarpes(h5f, file_path):
         energies,
         data,
         scan_type,
-        params[key],
+        params['hv'],
         params['deflector'],
         analyzer,
         file_note,
@@ -1050,7 +1050,7 @@ def load_nxarpes(h5f, file_path):
 
 
 def load_nxarpes_simulated(h5f, file_path):
-    """Load NXARPES example
+    """Load NXARPES example.
 
     Args:
         h5f: The HDF5 file-object from h5py.File.
@@ -1058,7 +1058,6 @@ def load_nxarpes_simulated(h5f, file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     analyzer = NavAnalyzer()
     analyzer._set_def_lorea_alba()
 
@@ -1100,7 +1099,7 @@ def load_nxarpes_simulated(h5f, file_path):
 
 
 def load_nxarpes_generic(h5f, file_path):
-    """Load generic NXARPES as saved by save_nxarpes_generic
+    """Load generic NXARPES as saved by save_nxarpes_generic.
 
     Args:
         h5f: The HDF5 file-object from h5py.File.
@@ -1160,7 +1159,7 @@ def load_nxarpes_generic(h5f, file_path):
 
 
 def load_nxsantares(h5f, file_path):
-    """Load NEXUS file from Antares/Soleil(FR)
+    """Load NEXUS file from Antares/Soleil(FR).
 
     Args:
         h5f: The HDF5 file-object from h5py.File.
@@ -1168,7 +1167,6 @@ def load_nxsantares(h5f, file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
     analyzer = NavAnalyzer()
     analyzer._set_def_antares_soleil()
@@ -1176,10 +1174,7 @@ def load_nxsantares(h5f, file_path):
     fst_grp = list(h5f.keys())[0]
 
     # deflector angles
-    if not analyzer.deflector:
-        defl_angles = 0
-    else:
-        defl_angles = h5f[fst_grp+"/scan_data/actuator_1_1"][()]
+    defl_angles = h5f[fst_grp+"/scan_data/actuator_1_1"][()]
 
     # scans and scan_type
     scans = defl_angles
@@ -1247,7 +1242,7 @@ def load_nxsantares(h5f, file_path):
 
 
 def load_cassiopee(file_path):
-    """Load ARPES data from Cassiopee/Soleil(FR)
+    """Load ARPES data from Cassiopee/Soleil(FR).
 
     Args:
         path: file path of a ROI-file in the folder.
@@ -1255,14 +1250,12 @@ def load_cassiopee(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
     analyzer = NavAnalyzer()
     analyzer._set_def_cassiopee_soleil()
 
     # deflector angles
-    if not analyzer.deflector:
-        defl_angles = 0
+    defl_angles = 0
 
     file_dir = os.path.abspath(os.path.dirname(file_path))
 
@@ -1414,9 +1407,8 @@ def load_scienta_ses_zip(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
-    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5, deflector=True)
+    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5)
 
     # Load the zip file
     entry_zip = np.load(file_path)
@@ -1506,7 +1498,6 @@ def load_known_txt(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # Select the correct loading function depending on the first line
     with open(file_path) as fdat:
         line = fdat.readline()
@@ -1528,7 +1519,6 @@ def load_scienta_or_mbs_txt(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # Load the txt-file
     params = {}
     with open(file_path) as fdat:
@@ -1558,13 +1548,17 @@ def load_scienta_or_mbs_txt(file_path):
             energies = np.array(
                 params["Dimension 1 scale"].replace(',', '.').split()
             ).astype(np.float)
-            hv = np.array([
-                float(params['Excitation Energy'].replace(',', '.'))])
+            if 'Excitation Energy' in params:
+                hv = np.array([
+                    float(params['Excitation Energy'].replace(',', '.'))])
+            else:
+                # WARNING: can't find photon energy (hv), using default of 123
+                hv = np.array([123])
 
             len_ang = len(angles)
             len_en = len(energies)
 
-            if "Dimension 3 size":
+            if "Dimension 3 size" in params:
                 defl_angles = np.array(
                     params["Dimension 3 scale"].replace(',', '.').split()
                 ).astype(np.float)
@@ -1660,7 +1654,7 @@ def load_specs_sp2(file_path):
         NavEntry (class): the class for the data to be explored by NavARP.
     """
     # analyzer
-    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5, deflector=False)
+    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5)
 
     # Load the sp2-file
     params = {}
@@ -1722,33 +1716,48 @@ def load_mbs_krx(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     # analyzer
-    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.6, deflector=False)
+    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.6)
 
     file_open = open(file_path, 'rb')
 
     # first 4-bytes integer is 3-times the number of images
     len_scan = int.from_bytes(file_open.read(4), "little")//3
+    file_starts = int.from_bytes(file_open.read(4), "little")
+    if file_starts == 0:
+        is64bit = True
+    else:
+        is64bit = False
 
+    # going back before reading the file_start
+    file_open.seek(4)
     # the point array and size (pas) of the images are in the following bytes
     nums = len_scan*3
-    pas = np.frombuffer(file_open.read(nums*4), dtype='<i', count=nums)
+    if is64bit:
+        pas = np.frombuffer(file_open.read(nums*16), dtype='<i', count=nums*2)
+        pas = pas[1::2]
+    else:
+        pas = np.frombuffer(file_open.read(nums*4), dtype='<i', count=nums)
 
     # taking the size from the first image, assuming it is the same for all
     len_ang, len_en = pas[1:3]
 
-    # getting the header byte length wich is after each image
-    header_len = pas[3] - pas[0] - (len_en*len_ang) - 1
-
     # creating the arrays with the correct size
     data = np.zeros((len_scan, len_ang, len_en), dtype='float32')
     # assuming the scan is a deflector scan
-    scan_type = 'deflector'
     defl_angles = np.zeros(len_scan)
 
+    # getting the header byte length wich is after each image
+    if len_scan == 1:
+        scan_type = 'single'
+        file_open.seek(0, 2)
+        eof_pos = file_open.tell()
+        header_len = eof_pos - pas[0] - (len_en*len_ang) - 1
+    else:
+        scan_type = 'deflector'
+        header_len = pas[3] - pas[0] - (len_en*len_ang) - 1
     # read first header only
-    file_open.seek((pas[0]+(len_en*len_ang))*4 + 4)
+    file_open.seek((pas[0] + len_en*len_ang + 1)*4)
     header = file_open.read(header_len*4)
     params = {}
     line_dec = header.decode("utf-8")
@@ -1757,26 +1766,44 @@ def load_mbs_krx(file_path):
         if len(linattr_sp) > 1:
             params[linattr_sp[0]] = linattr_sp[1]
 
-    x_scale = np.linspace(
-        float(params['XScaleMin']),
-        float(params['XScaleMax']),
-        int(params['NoS'])
-    )
     e_kin = np.linspace(
         float(params['Start K.E.']),
         float(params['End K.E.']),
         int(params['No. Steps'])
     )
-    defl_angles = np.linspace(
-        float(params['YScaleMin']),
-        float(params['YScaleMax']),
-        int(len_scan)
-    )
+
+    if is64bit:
+        x_scale = np.linspace(
+            float(params['ScaleMin']),
+            float(params['ScaleMax']),
+            int(params['NoS'])
+        )
+        if len_scan != 1:
+            defl_angles = np.linspace(
+                float(params['MapStartX']),
+                float(params['MapEndX']),
+                int(len_scan)
+            )
+    else:
+        x_scale = np.linspace(
+            float(params['XScaleMin']),
+            float(params['XScaleMax']),
+            int(params['NoS'])
+        )
+        if len_scan != 1:
+            defl_angles = np.linspace(
+                float(params['YScaleMin']),
+                float(params['YScaleMax']),
+                int(len_scan)
+            )
 
     # reading all the images
     for i in range(len_scan):
         # going in the image position using the pointer
-        file_open.seek(pas[i*3]*4)
+        if is64bit:
+            file_open.seek(pas[i*3]*4)
+        else:
+            file_open.seek(pas[i*3]*4)
         data[i, :, :] = np.frombuffer(
             file_open.read((len_en*len_ang)*4),
             dtype='<i4',
@@ -1829,7 +1856,7 @@ def load_igorpro_pxt(file_path):
         NavEntry (class): the class for the data to be explored by NavARP.
     """
     # analyzer
-    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5, deflector=False)
+    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5)
 
     # Load the pxt-file
     pxt_file = igor.load(file_path)
@@ -1858,7 +1885,12 @@ def load_igorpro_pxt(file_path):
         if '=' in string:
             string_split = string.split('=')
             params[string_split[0]] = string_split[1]
-    hv = np.array([float(params['Excitation Energy'].replace(',', '.'))])
+
+    if 'Excitation Energy' in params:
+        hv = np.array([float(params['Excitation Energy'].replace(',', '.'))])
+    else:
+        # WARNING: can't find photon energy (hv), using default value of 123
+        hv = np.array([123])
 
     scans = np.array([1])
 
@@ -1885,7 +1917,7 @@ def load_igorpro_ibw(file_path):
         NavEntry (class): the class for the data to be explored by NavARP.
     """
     # analyzer
-    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5, deflector=False)
+    analyzer = NavAnalyzer(tht_ap=50, phi_ap=0, work_fun=4.5)
 
     # Load the pxt-file
     ibw_file = binarywave.load(file_path)
@@ -1911,7 +1943,11 @@ def load_igorpro_ibw(file_path):
         if '=' in string:
             string_split = string.split('=')
             params[string_split[0]] = string_split[1]
-    hv = np.array([float(params['Excitation Energy'].replace(',', '.'))])
+    if 'Excitation Energy' in params:
+        hv = np.array([float(params['Excitation Energy'].replace(',', '.'))])
+    else:
+        # WARNING: can't find photon energy (hv), using default value of 123
+        hv = np.array([123])
 
     scans = np.array([1])
 
@@ -1929,7 +1965,7 @@ def load_igorpro_ibw(file_path):
 
 
 def load_igorpro_itx(file_path):
-    """Method for loading ARPES data from itx file
+    """Load ARPES data from itx file.
 
     Args:
         path: file path of the itx-file.
@@ -1937,7 +1973,6 @@ def load_igorpro_itx(file_path):
     Returns:
         NavEntry (class): the class for the data to be explored by NavARP.
     """
-
     file_note = ''
     with open(file_path) as fdat:
         fdat.readline()  # IGOR
@@ -2068,18 +2103,26 @@ def load_navarp_yaml(file_path):
             os.path.join(file_data_dir, fname) for fname in file_names]
 
         data = None
+        ekin = None
         for i, file_path in enumerate(file_paths):
             entry_i = load(file_path)
             if data is not None:
                 data[i, :, :] = entry_i.data
+                ekin[i, :] = entry_i.energies
             else:
                 data_0 = entry_i.data
                 data = np.zeros(
                     (len(file_paths), data_0.shape[1], data_0.shape[2]))
                 data[0, :, :] = data_0
+
+                ekin_0 = entry_i.energies
+                ekin = np.zeros((len(file_paths), ekin_0.shape[0]))
+                ekin[0, :] = ekin_0
         entry_out = entry_i
         entry_out.data = data
+        entry_out.energies = ekin
     else:
+        print(file_data_path)
         file_path = file_data_path
         entry_out = load(file_path)
 
@@ -2122,10 +2165,14 @@ def load_navarp_yaml(file_path):
             if attr in analyzer_info:
                 setattr(entry_out.analyzer, attr, analyzer_info[attr])
 
-    if entry_out.scan_type == 'deflector':
-        entry_out.defl_angles = entry_out.scans
-    elif entry_out.scan_type == 'hv':
+    if entry_out.scan_type == 'hv':
         entry_out.hv = entry_out.scans
+    else:
+        # without checking the consistency, use as energies only the first one
+        # since the kinetic energies cannot be different without a hv scan
+        entry_out.energies = entry_out.energies[0, :]
+        if entry_out.scan_type == 'deflector':
+            entry_out.defl_angles = entry_out.scans
 
     entry_out.file_note = re.sub(
         "scan_type = [A-Za-z_]+",
@@ -2136,10 +2183,60 @@ def load_navarp_yaml(file_path):
     entry_out.efermi = entry_out.hv - entry_out.analyzer.work_fun
     entry_out.init_ebins()
 
-    if 'tht_an' in entry_info:
-        entry_out.tht_an = entry_info['tht_an']
-    else:
-        entry_out.tht_an = None
+    if 'set_tht_an' in entry_info:
+        # check if all necessary args are present in the yaml file
+        set_tht_an_keys = ['tht_p', 'k_along_slit_p', 'e_kin_p']
+        if all(key in entry_info['set_tht_an'] for key in set_tht_an_keys):
+
+            # read all the optional args if present in the yaml file
+            kargs = {'p_hv': False, 'hv_p': None, 'print_out': True}
+            for key in kargs:
+                if key in entry_info['set_tht_an']:
+                    kargs[key] = entry_info['set_tht_an'][key]
+
+            entry_out.set_tht_an(
+                entry_info['set_tht_an']['tht_p'],
+                entry_info['set_tht_an']['k_along_slit_p'],
+                entry_info['set_tht_an']['e_kin_p'],
+                kargs['p_hv'],
+                kargs['hv_p'],
+                kargs['print_out']
+            )
+        else:
+            print("WARNING: impossible to set_tht_an, args missing.")
+
+    if 'set_kspace' in entry_info:
+        # check if all necessary args are present in the yaml file
+        set_kspace_keys = [
+            'tht_p', 'k_along_slit_p', 'scan_p', 'ks_p', 'e_kin_p']
+        if all(key in entry_info['set_kspace'] for key in set_kspace_keys):
+
+            # read all the optional args if present in the yaml file
+            kargs = {
+                'inn_pot': 14,
+                'p_hv': False,
+                'hv_p': None,
+                'k_perp_slit_for_kz': 0,
+                'print_out': True
+            }
+            for key in kargs:
+                if key in entry_info['set_kspace']:
+                    kargs[key] = entry_info['set_kspace'][key]
+
+            entry_out.set_kspace(
+                entry_info['set_kspace']['tht_p'],
+                entry_info['set_kspace']['k_along_slit_p'],
+                entry_info['set_kspace']['scan_p'],
+                entry_info['set_kspace']['ks_p'],
+                entry_info['set_kspace']['e_kin_p'],
+                kargs['inn_pot'],
+                kargs['p_hv'],
+                kargs['hv_p'],
+                kargs['k_perp_slit_for_kz'],
+                kargs['print_out']
+            )
+        else:
+            print("WARNING: impossible to set_kspace, args missing.")
 
     if 'efermi' in entry_info:
         efermi = np.array(entry_info['efermi'])
@@ -2149,7 +2246,7 @@ def load_navarp_yaml(file_path):
 
 
 def save_nxarpes_generic(entry, file_path_nxs, instrument_name=r"unknow"):
-    """The function saves NavEntry into file_path as generic NXARPES.
+    """Save NavEntry into file_path as generic NXARPES.
 
     Args:
         entry (NavEntry class): entry to be saved.
@@ -2157,7 +2254,6 @@ def save_nxarpes_generic(entry, file_path_nxs, instrument_name=r"unknow"):
         instrument_name (string, optional): name of the instrument where the
             data are from (e.g.: beamline or laboratory name).
     """
-
     f = h5py.File(file_path_nxs, "w")  # create the HDF5 NeXus file
     f.attrs[u"default"] = u"entry1"
 
