@@ -283,10 +283,14 @@ def norm(z, mode="all", axis=1):
             z_norm = np.sum(z_norm, axis=axis)/(z_norm.shape[axis])
             z_norm = np.transpose(np.tile(z_norm, (z.shape[axis], 1)))
         z_norm = z/z_norm
-        z_norm = (z_norm-z_norm.min())/(z_norm.max()-z_norm.min())
+        z_norm_min = np.nanmin(z_norm)
+        z_norm_max = np.nanmax(z_norm)
+        z_norm = (z_norm-z_norm_min)/(z_norm_max-z_norm_min)
     elif mode == "all":
         # normalize the matrix by its maximun
-        z_norm = (z-z.min())/(z.max()-z.min())
+        z_min = np.nanmin(z)
+        z_max = np.nanmax(z)
+        z_norm = (z-z_min)/(z_max-z_min)
     else:
         print('Value Error: mode "{}" not understood,'
               ' mode can be only "all" or "each".'.format(mode))
